@@ -1,5 +1,5 @@
 /*
-Copyright (C) 1997-2011 Erwin Waterlander
+Copyright (C) 1997-2013 Erwin Waterlander
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -34,6 +34,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "nameset.h"
 #include "config.h"
 #include "dosdir.h"
+#include "graphics.h"
 
 #include <stdarg.h>
 #if defined(WCD_WINDOWS)
@@ -158,7 +159,7 @@ size_t str_columns (char *s)
       return(strlen(s));
    else
    {
-      j = wcswidth(wstr,(size_t)DD_MAXPATH);
+      j = wcd_wcswidth(wstr,(size_t)DD_MAXPATH);
       /* j =  nr. of columns */
       if ( j < 0)
          return(strlen(s));
@@ -935,18 +936,23 @@ void printLine(WINDOW *win, nameset n, int i, int y, int xoffset, int *use_numbe
          j = 0;
          while ((j<(int)len)&&(c<xoffset))
          {
-            if (wcwidth(wstr[j]) != 0 )
+            if (wcd_wcwidth(wstr[j]) != 0 )
                c++;
             j++; /* j advances over combining characters */
          }
-         while ((j<(int)len)&&(wcwidth(wstr[j]) == 0 ))  /* Skip combining characters */
+#if defined(_WIN32) || defined(__CYGWIN__)
+         /* Line starts with a trailing surrogate character, insert a space */
+	 if ((wstr[j] >= 0xdc00) && (wstr[j] < 0xe000))
+           wstr[j] = ' ';
+#endif
+         while ((j<(int)len)&&(wcd_wcwidth(wstr[j]) == 0 ))   /* Skip combining characters */
            j++;
-         width = wcwidth(wstr[j]);
+         width = wcd_wcwidth(wstr[j]);
          while ((j<(int)len)&&((nr_offset+width)<(COLS-1)))
          {
             waddnwstr(win,wstr+j,1);
             j++;
-            width = width + wcwidth(wstr[j]);
+            width = width + wcd_wcwidth(wstr[j]);
          }
       }
 #else
@@ -1001,20 +1007,25 @@ void printStackLine(WINDOW *win, WcdStack ws, int i, int y, int xoffset, int *us
          j = 0;
          while ((j<(int)len)&&(c<xoffset))
          {
-            if (wcwidth(wstr[j]) != 0 )
+            if (wcd_wcwidth(wstr[j]) != 0 )
                c++;
             j++; /* j advances over combining characters */
          }
-         while ((j<(int)len)&&(wcwidth(wstr[j]) == 0 ))  /* Skip combining characters */
+#if defined(_WIN32) || defined(__CYGWIN__)
+         /* Line starts with a trailing surrogate character, insert a space */
+	 if ((wstr[j] >= 0xdc00) && (wstr[j] < 0xe000))
+           wstr[j] = ' ';
+#endif
+         while ((j<(int)len)&&(wcd_wcwidth(wstr[j]) == 0 ))   /* Skip combining characters */
            j++;
-         width = wcwidth(wstr[j]);
+         width = wcd_wcwidth(wstr[j]);
          while ((j<(int)len)&&((nr_offset+width)<(COLS-1)))
          {
             waddnwstr(win,wstr+j,1);
             j++;
-            width = width + wcwidth(wstr[j]);
+            width = width + wcd_wcwidth(wstr[j]);
          }
-         if ((i == ws->current) && ((nr_offset+width-wcwidth(wstr[j])+2)<(COLS-1)))
+         if ((i == ws->current) && ((nr_offset+width-wcd_wcwidth(wstr[j])+2)<(COLS-1)))
             wprintw(win," *");
       }
 #else
@@ -1075,7 +1086,7 @@ void print_list_stack(WINDOW *scrollWin, int lines_per_page,int line, WcdStack w
 
 void print_list(WINDOW *scrollWin, int lines_per_page,int line, nameset list, WcdStack ws, int start, int top, int bottom, int use_numbers, int xoffset)
 {
-   wclear(scrollWin);
+   werase(scrollWin);
    if (list != NULL)
       print_list_normal(scrollWin,lines_per_page,line,list,top,bottom,use_numbers,xoffset);
    else
@@ -1092,8 +1103,8 @@ void displayRefresh(int init)
   int page, pageoffset, offset, n;
   char buf[WCD_MAX_INPSTR];
 
-  wclear(wcd_display.scrollWin);
-  wclear(wcd_display.inputWin);
+  werase(wcd_display.scrollWin);
+  werase(wcd_display.inputWin);
 
 
  if (init || (wcd_display.bottom >= wcd_display.size) || (wcd_display.size <= wcd_display.lines_per_page)) {
@@ -1144,7 +1155,7 @@ void displayRefresh(int init)
 }
 void displayHelp(WINDOW *win, int height)
 {
-   wclear(win);
+   werase(win);
    if (height < 17)
       wcd_mvwaddstr(win,0,0,_("Screenheight must be > 20 for help."));
    else
