@@ -26,10 +26,11 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #endif
 
 #ifdef WCD_USECURSES
+#  include <curses.h>
+/* MOUSE_MOVED in PDCurses' curses.h conflicts with MOUSE_MOVED in wincon.h */
 #  ifdef MOUSE_MOVED
 #    undef MOUSE_MOVED
 #  endif
-#  include <curses.h>
 #endif
 
 size_t wcstoansi(char *mbstr, const wchar_t *wcstr, int len);
@@ -37,6 +38,7 @@ size_t ansitowcs(wchar_t *wcstr, const char *mbstr, int len);
 size_t wcstoutf8(char *mbstr, const wchar_t *wcstr, int len);
 size_t utf8towcs(wchar_t *wcstr, const char *mbstr, int len);
 void wcd_printf( const char* format, ... );
+void wcd_wprintf( const wchar_t* format, ... );
 size_t str_columns (char *s);
 void sort_list(nameset list);
 int display_list(nameset list,int perfect, int use_numbers, int use_stdout);
