@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000-2011 Erwin Waterlander
+Copyright (C) 2000-2013 Erwin Waterlander
 
 Ideas and source code of NCD (Ninux Czo Directory) have been
 used in the WCD graphical interface.
@@ -36,11 +36,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
-#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
+#if defined(WCD_UNICODE) || (defined(_WIN32) && !defined(__CYGWIN__))
 #ifndef __USE_XOPEN
 #define __USE_XOPEN
 #endif
 #include <wchar.h>
+#include "wcwidth.h" /* Marcus Kuhn's wcwidth implementation */
 #endif
 #include "std_macr.h"
 #include "structur.h"
@@ -83,6 +84,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 # define WCD_SEL_ON 6           /* selection on/off codes */
 # define WCD_SEL_OFF 7
 
+# define WCD_SPACE 8            /* double width space in CJK mode */
+
 #ifdef ASCII_TREE
 static const char WCD_ONESUBDIR[] = "---" ;
 static const char WCD_SPLITDIR[] =  "-+-" ;
@@ -90,13 +93,19 @@ static const char WCD_SUBDIR[] =   "  |-" ;
 static const char WCD_MOREDIR[] =  "  |  " ;
 static const char WCD_ENDDIR[] =   "  `-" ;
 static const char WCD_OVERDIR[] =  "     " ;
+static const char WCD_COMPACT_SUBDIR[] =   " |--" ;
+static const char WCD_COMPACT_MOREDIR[] =  " |  " ;
+static const char WCD_COMPACT_ENDDIR[] =   " `--" ;
 #else
 static const char WCD_ONESUBDIR[] = { WCD_ACS_HL, WCD_ACS_HL, WCD_ACS_HL, 0} ;
 static const char WCD_SPLITDIR[] =  { WCD_ACS_HL, WCD_ACS_TT, WCD_ACS_HL, 0} ;
-static const char WCD_SUBDIR[] =    { 32, 32, WCD_ACS_LT, WCD_ACS_HL, 0} ;
-static const char WCD_MOREDIR[] =   { 32, 32, WCD_ACS_VL, 32, 32, 0} ;
-static const char WCD_ENDDIR[] =    { 32, 32, WCD_ACS_LLC, WCD_ACS_HL, 0} ;
-static const char WCD_OVERDIR[] =  "     " ;
+static const char WCD_SUBDIR[] =    { 32,  WCD_SPACE,  WCD_ACS_LT,  WCD_ACS_HL, 0} ;
+static const char WCD_MOREDIR[] =   { 32,  WCD_SPACE,  WCD_ACS_VL,  WCD_SPACE, 32, 0} ;
+static const char WCD_ENDDIR[] =    { 32,  WCD_SPACE,  WCD_ACS_LLC, WCD_ACS_HL, 0} ;
+static const char WCD_OVERDIR[] =   { 32,  WCD_SPACE,  WCD_SPACE, WCD_SPACE,   32, 0} ;
+static const char WCD_COMPACT_SUBDIR[] =    { 32, WCD_ACS_LT, WCD_ACS_HL, WCD_ACS_HL, 0} ;
+static const char WCD_COMPACT_MOREDIR[] =   { 32, WCD_ACS_VL, WCD_SPACE,  WCD_SPACE, 0} ;
+static const char WCD_COMPACT_ENDDIR[] =    { 32, WCD_ACS_LLC, WCD_ACS_HL, WCD_ACS_HL, 0} ;
 #endif
 
 #define WCD_GRAPH_MAX_LINE_LENGTH  DD_MAXPATH * 2
@@ -187,10 +196,40 @@ c3po_bool dirHasSubdirs(dirnode d)
 }
 
 
+char* getCompactTreeLine(dirnode d, int y, int *y_orig, char *line, char *tline)
+{
+   dirnode n;
+
+   if (dirnodeHasParent(d) eq false)
+      return(line);
+
+   if (y == *y_orig)
+   {
+      if (dirnodeGetDown(d) == NULL)
+         strcpy(tline,WCD_COMPACT_ENDDIR);
+      else
+         strcpy(tline,WCD_COMPACT_SUBDIR);
+      if (d->fold eq true)
+         tline[strlen(tline)-1] = '+';
+
+   }
+   else
+   {
+      if (dirnodeGetDown(d) == NULL)
+         strcpy(tline,"    ");
+      else
+         strcpy(tline,WCD_COMPACT_MOREDIR);
+   }
+   strcat(tline,line);
+   strcpy(line,tline);
+
+   n = dirnodeGetParent(d);
+   return(getCompactTreeLine(n,dirnodeGetY(n),y_orig, line, tline));
+}
 
 /* this function is called by other functions only on directories that are last
  * on a line */
-char* getTreeLine(dirnode d, int y, int *y_orig, dirnode curNode, c3po_bool fold)
+char* getTreeLine(dirnode d, int y, int *y_orig, dirnode curNode, c3po_bool fold, const int *graphics_mode)
 {
    static text line = NULL;
    static text tline = NULL;
@@ -214,6 +253,22 @@ char* getTreeLine(dirnode d, int y, int *y_orig, dirnode curNode, c3po_bool fold
       tline[0] = '\0';
    }
 
+   if (*graphics_mode & WCD_GRAPH_COMPACT) /* compact tree */
+   {
+      strcpy(tline," ");
+      if (d == curNode)
+         tline[strlen (tline) - 1] = WCD_SEL_ON;
+
+      strcat(tline,dirnodeGetName(d));
+
+      strcat(tline," ");
+      if (d == curNode)
+         tline[strlen (tline) - 1] = WCD_SEL_OFF;
+
+      strcpy(line,tline);
+      getCompactTreeLine(d,dirnodeGetY(d),y_orig, line, tline);
+      return line;
+   }
    /* buffer overflow prevention:
     *
     * The max length of `line' and `tline' is the length of `line' increased
@@ -343,7 +398,7 @@ char* getTreeLine(dirnode d, int y, int *y_orig, dirnode curNode, c3po_bool fold
    if (dirnodeHasParent(d) eq true)
    {
       n = dirnodeGetParent(d);
-      return(getTreeLine(n,dirnodeGetY(d),y_orig,curNode,d->fold));
+      return(getTreeLine(n,dirnodeGetY(d),y_orig,curNode,d->fold, graphics_mode));
    }
 
    if (*y_orig != 0)
@@ -356,30 +411,13 @@ char* getTreeLine(dirnode d, int y, int *y_orig, dirnode curNode, c3po_bool fold
    return line;
 }
 
-void dumpTree(dirnode d)
+void dumpTreeLine(dirnode d, const int *graphics_mode)
 {
-   size_t index, size;
-   dirnode n;
    int y;
    char *l;
 
-
-   index = 0;
-
-   if(isEmptyDirnode(d) eq false)
-   {
-         size = getSizeOfDirnode(d);
-         while(index < size)
-         {
-            n = elementAtDirnode(index,d);
-            dumpTree(n);
-            index++;
-         }
-   }
-   else
-   {
       y = dirnodeGetY(d);
-      l = getTreeLine(d,y,&y,NULL,false);
+      l = getTreeLine(d,y,&y,NULL,false, graphics_mode);
       while ( *l != '\0')
          {
             switch(*l)
@@ -405,12 +443,47 @@ void dumpTree(dirnode d)
                case WCD_SEL_OFF:
                 putc('>', stdout);
                   break;
+               case WCD_SPACE:
+                putc(' ', stdout);
+                  break;
                default:
                 putc(*l, stdout);
             }
             l++;
          }
       printf("\n");
+}
+void dumpTree(dirnode d, const int *graphics_mode)
+{
+   size_t index, size;
+   dirnode n;
+
+
+   index = 0;
+   if (*graphics_mode & WCD_GRAPH_COMPACT)
+   {
+         dumpTreeLine(d, graphics_mode);
+         size = getSizeOfDirnode(d);
+         while(index < size)
+         {
+            n = elementAtDirnode(index,d);
+            dumpTree(n, graphics_mode);
+            index++;
+         }
+   }
+   else if (isEmptyDirnode(d) eq false)
+   {
+         size = getSizeOfDirnode(d);
+         while(index < size)
+         {
+            n = elementAtDirnode(index,d);
+            dumpTree(n, graphics_mode);
+            index++;
+         }
+   }
+   else
+   {
+         dumpTreeLine(d, graphics_mode);
    }
 
 
@@ -421,7 +494,7 @@ void dumpTree(dirnode d)
  * in the tree
  */
 
-void setXYTree(dirnode d)
+void setXYTree(dirnode d, const int *graphics_mode)
 {
    size_t index, len, size;
    int x;
@@ -431,22 +504,52 @@ void setXYTree(dirnode d)
 
    if(dirHasSubdirs(d) eq true)
    {
-      len = str_columns(dirnodeGetName(d));
-      y = dirnodeGetY(d);
-      index = 0;
-      size = getSizeOfDirnode(d);
-      while(index < size)
+      if (*graphics_mode & WCD_GRAPH_COMPACT) /* compact tree */
       {
-         n = elementAtDirnode(index,d);
-         x = dirnodeGetX(d) + (int)len + 5;
-         dirnodeSetX(x,n);
-         dirnodeSetY(y,n);
-         setXYTree(n);
-         index++;
-         y = y + 1;
+         len = str_columns(dirnodeGetName(d));
+         y = dirnodeGetY(d);
+         index = 0;
+         size = getSizeOfDirnode(d);
+         while(index < size)
+         {
+            y = y + 1;
+            n = elementAtDirnode(index,d);
+#ifndef ASCII_TREE
+            if ((*graphics_mode & WCD_GRAPH_CJK) && !(*graphics_mode & WCD_GRAPH_ASCII)) /* double width line drawing symbols. */
+               x = dirnodeGetX(d) + 7;
+            else
+#endif
+               x = dirnodeGetX(d) + 4;
+            dirnodeSetX(x,n);
+            dirnodeSetY(y,n);
+            setXYTree(n, graphics_mode);
+            index++;
+         }
+      }
+      else
+      {
+         len = str_columns(dirnodeGetName(d));
+         y = dirnodeGetY(d);
+         index = 0;
+         size = getSizeOfDirnode(d);
+         while(index < size)
+         {
+            n = elementAtDirnode(index,d);
+#ifndef ASCII_TREE
+            if ((*graphics_mode & WCD_GRAPH_CJK) && !(*graphics_mode & WCD_GRAPH_ASCII)) /* double width line drawing symbols. */
+               x = dirnodeGetX(d) + (int)len + 8;
+            else
+#endif
+               x = dirnodeGetX(d) + (int)len + 5;
+            dirnodeSetX(x,n);
+            dirnodeSetY(y,n);
+            setXYTree(n, graphics_mode);
+            index++;
+            y = y + 1;
+         }
+         /* y is 1 too large */
+         y = y - 1;
       }
-      /* y is 1 too large */
-      y = y - 1;
    }
 }
 
@@ -480,7 +583,7 @@ void addPath(text path, dirnode d)
    dirnode n,n_up;
    text t;
    size_t index;
-#if (defined(WIN32) || defined(__CYGWIN__))
+#if (defined(_WIN32) || defined(__CYGWIN__))
    static char buf[DD_MAXPATH] = "//" ;
 
    if ( (path != NULL) &&
@@ -612,7 +715,7 @@ dirnode searchNodeForDir(text path, dirnode d, dirnode rNode)
    dirnode n;
    size_t index;
    dirnode rootnode;
-#if (defined(WIN32) || defined(__CYGWIN__))
+#if (defined(_WIN32) || defined(__CYGWIN__))
    static char buf[DD_MAXPATH] = "//" ;
 #endif
 
@@ -625,7 +728,7 @@ dirnode searchNodeForDir(text path, dirnode d, dirnode rNode)
       return searchNodeForDir(path, rootnode, rootnode); /* start on top of tree */
    }
 
-#if (defined(WIN32) || defined(__CYGWIN__))
+#if (defined(_WIN32) || defined(__CYGWIN__))
    if ( (path != NULL) &&
         (*path == '/') &&
         (*(path+1) == '/') )
@@ -815,12 +918,17 @@ dirnode getFirstNodeInLevel(dirnode node, int level)
 }
 
 /******************************************************/
+/* get last node (most right side) on a line */
 
 dirnode getLastNodeInSameLevel(dirnode node)
 {
    if (node == NULL)
       return(NULL);
 
+   /* In compact mode there is only one node per line */
+   if (wcd_cwin.graphics_mode & WCD_GRAPH_COMPACT)
+      return(node);
+
    if (dirHasSubdirs(node) eq false)
       return(node);
    else
@@ -887,10 +995,19 @@ dirnode getNodeCursDown(dirnode curNode)
  * If node is a valid pointer to a node this function
  * will return always a node.
  ******************************************************/
-dirnode getNodeCursLeft(dirnode curNode)
+dirnode getNodeCursLeft(dirnode curNode, int *ymax)
 {
    dirnode node;
 
+   if ((wcd_cwin.graphics_mode & WCD_GRAPH_COMPACT) && (wcd_cwin.graphics_mode & WCD_GRAPH_ALT))
+   {
+      if((dirnodeHasSubdirs(curNode) eq true) && (curNode->fold eq false))
+      {
+         setFold(wcd_cwin.curNode,true,ymax); /* fold */
+         return(curNode);
+      }
+   }
+
    if ((node = Left(curNode)) == NULL)
       return(curNode); /* there is no Left node, return same node */
 
@@ -1226,7 +1343,47 @@ dirnode findDirInCicle(char *dir, dirnode curNode, int exact, int ignore_case, i
  *
  * **********************************************************/
 
+#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
+int wcd_wcwidth(wchar_t c)
+{
+    switch(c)
+    {
+       case WCD_ACS_HL:
+       case WCD_ACS_VL:
+       case WCD_ACS_LT:
+       case WCD_ACS_LLC:
+       case WCD_ACS_TT:
+       case WCD_SPACE:
+#ifndef ASCII_TREE
+          if ((wcd_cwin.graphics_mode & WCD_GRAPH_CJK) && !(wcd_cwin.graphics_mode & WCD_GRAPH_ASCII))
+             return(2);
+          else
+#endif
+             return(1);
+          break;
+       case WCD_SEL_ON:
+       case WCD_SEL_OFF:
+          return(1);
+          break;
+       default:
+          if (wcd_cwin.graphics_mode & WCD_GRAPH_CJK)
+             return(mk_wcwidth_cjk(c));
+          else
+             return(wcwidth(c));
+    }
+}
+
+int wcd_wcswidth(const wchar_t *pwcs, size_t n)
+{
+    if (wcd_cwin.graphics_mode & WCD_GRAPH_CJK)
+       return(mk_wcswidth_cjk(pwcs, n));
+    else
+       return(wcswidth(pwcs, n));
+}
+#endif
+
 #ifdef WCD_USECURSES
+
 void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffset)
 {
    wcd_uchar *s;
@@ -1237,7 +1394,7 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
    int width, c;
 #endif
 
-   s = (wcd_uchar *)getTreeLine(getLastNodeInLevel(n,i),i,&i,curNode,false);
+   s = (wcd_uchar *)getTreeLine(getLastNodeInLevel(n,i),i,&i,curNode,false, &wcd_cwin.graphics_mode);
 
    if (s != NULL)
    {
@@ -1294,6 +1451,13 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
                 else
                   waddch (win,ACS_TTEE);
                   break;
+               case WCD_SPACE:
+                waddch(win, (chtype)' ');
+#ifndef ASCII_TREE
+                if ((wcd_cwin.graphics_mode & WCD_GRAPH_CJK) && !(wcd_cwin.graphics_mode & WCD_GRAPH_ASCII))
+                  waddch(win, (chtype)' ');
+#endif
+                  break;
                case WCD_SEL_ON:
                 wattron(win,A_REVERSE);
                 waddch(win,(chtype)'['); /* square brackets indicate there was an error. */
@@ -1313,49 +1477,34 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
          j = 0;
          while ((j<(int)len)&&(c<xoffset))
          {
-            switch(wstr[j])
-            {
-               case WCD_ACS_HL:
-               case WCD_ACS_VL:
-               case WCD_ACS_LT:
-               case WCD_ACS_LLC:
-               case WCD_ACS_TT:
-               case WCD_SEL_ON:
-               case WCD_SEL_OFF:
-                  c = c + 1;
-                  break;
-               default:
-                  c = c + wcwidth(wstr[j]);
-            }
+            c = c + wcd_wcwidth(wstr[j]);
             j++; /* j advances over combining characters */
          }
          /* xoffset is horizontal offset measured in nr. of columns.
           * This may be at the middle of a double width character. */
-         if (( c > xoffset ) && ( wcwidth(wstr[j-1]) == 2))
+         if (( c > xoffset ) && ( wcd_wcwidth(wstr[j-1]) == 2))
          {
             /* Last character skipped was a double width character.
              * Insert a space. */
             j--;
             wstr[j] = ' ';
          }
-         while ((j<(int)len)&&(wcwidth(wstr[j]) == 0 ))  /* Skip combining characters */
+#if defined(_WIN32) || defined(__CYGWIN__)
+         /* xoffset is horizontal offset measured in nr. of columns.
+          * This may be at the middle of a surrogate pair. */
+         if ((wstr[j] >= 0xdc00) && (wstr[j] < 0xe000))
+         {
+            /* Last character skipped was a surrogate lead.
+             * First character is a surrogate trail.
+             * Insert a space. */
+            wstr[j] = ' ';
+         }
+#endif
+         while ((j<(int)len)&&(wcd_wcwidth(wstr[j]) == 0 ))  /* Skip combining characters */
            j++;
          width = 0;
          if (j<(int)len)
-            switch(wstr[j])
-            {
-               case WCD_ACS_HL:
-               case WCD_ACS_VL:
-               case WCD_ACS_LT:
-               case WCD_ACS_LLC:
-               case WCD_ACS_TT:
-               case WCD_SEL_ON:
-               case WCD_SEL_OFF:
-                width = 1;
-                  break;
-               default:
-                width = wcwidth(wstr[j]);
-            }
+            width = wcd_wcwidth(wstr[j]);
          while ((j<(int)len)&&(width<COLS))
          {
             switch(wstr[j])
@@ -1390,6 +1539,13 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
                 else
                   waddch (win,ACS_TTEE);
                   break;
+               case WCD_SPACE:
+                waddch(win, (chtype)' ');
+#ifndef ASCII_TREE
+                if ((wcd_cwin.graphics_mode & WCD_GRAPH_CJK) && !(wcd_cwin.graphics_mode & WCD_GRAPH_ASCII))
+                  waddch(win, (chtype)' ');
+#endif
+                  break;
                case WCD_SEL_ON:
                 wattron(win,A_REVERSE);
                 waddch(win,(chtype)'<');
@@ -1402,20 +1558,7 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
                 waddnwstr(win,wstr+j,1);
             }
             j++;
-            switch(wstr[j])
-            {
-               case WCD_ACS_HL:
-               case WCD_ACS_VL:
-               case WCD_ACS_LT:
-               case WCD_ACS_LLC:
-               case WCD_ACS_TT:
-               case WCD_SEL_ON:
-               case WCD_SEL_OFF:
-                width = width + 1;
-                  break;
-               default:
-                width = width + wcwidth(wstr[j]);
-            }
+            width = width + wcd_wcwidth(wstr[j]);
          }
       }
 #else
@@ -1453,6 +1596,13 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
                 else
                   waddch (win,ACS_TTEE);
                break;
+            case WCD_SPACE:
+                waddch(win, (chtype)' ');
+#ifndef ASCII_TREE
+             if ((wcd_cwin.graphics_mode & WCD_GRAPH_CJK) && !(wcd_cwin.graphics_mode & WCD_GRAPH_ASCII))
+               waddch(win, (chtype)' ');
+#endif
+               break;
             case WCD_SEL_ON:
                   wattron(win,A_REVERSE);
                   waddch(win,(chtype)'<');
@@ -1523,8 +1673,8 @@ void dataRefresh(int ydiff, int init)
   if ((yposition < 0)|| init)
      yposition = (wcd_cwin.scrollWinHeight /2);
 
-  wclear(wcd_cwin.scrollWin);
-  wclear(wcd_cwin.inputWin);
+  werase(wcd_cwin.scrollWin);
+  werase(wcd_cwin.inputWin);
 
   if (ydiff < 0 )
   {
@@ -1589,12 +1739,12 @@ void dataRefresh(int ydiff, int init)
         waddch(wcd_cwin.inputWin, (chtype)s[i]);
    } else {
       i = 0;
-      width = wcwidth(wstr[i]);
+      width = wcd_wcwidth(wstr[i]);
       while ((i<len)&&(width<COLS))
       {
          waddnwstr(wcd_cwin.inputWin,wstr+i,1);
          i++;
-         width = width + wcwidth(wstr[i]);
+         width = width + wcd_wcwidth(wstr[i]);
       }
    }
 #else
@@ -1604,6 +1754,19 @@ void dataRefresh(int ydiff, int init)
 #endif
   }
 
+  /* Show a "C" when CJK width mode is on. */
+  if (wcd_cwin.graphics_mode & WCD_GRAPH_CJK)
+  {
+    wmove(wcd_cwin.inputWin, 2, COLS - 3);
+    waddstr(wcd_cwin.inputWin,"C");
+  }
+  /* Show an "A" when alternative navigation mode is on. */
+  if (wcd_cwin.graphics_mode & WCD_GRAPH_ALT)
+  {
+    wmove(wcd_cwin.inputWin, 2, COLS - 2);
+    waddstr(wcd_cwin.inputWin,"A");
+  }
+
   if (wcd_cwin.mode == WCD_NAV)
     wcd_mvwaddstr(wcd_cwin.inputWin, 2, 0, _("/ = search forward,  ? = search backward,  : = help"));
   else
@@ -1626,10 +1789,10 @@ void dataRefresh(int ydiff, int init)
 /************************************************************/
 void showHelp(WINDOW *win, int height)
 {
-   wclear(win);
+   werase(win);
    if (height > 21)
    {
-      wcd_mvwaddstr(win, 0,0,_("NAVIGATE MODE:"));
+      wcd_mvwaddstr(win, 0,0,_("NAVIGATE MODE (1/2):"));
       wcd_mvwaddstr(win, 1,0,_("h or <Left>       go left."));
       wcd_mvwaddstr(win, 2,0,_("j or <Down>       go down."));
       wcd_mvwaddstr(win, 3,0,_("k or <Up>         go up."));
@@ -1644,19 +1807,27 @@ void showHelp(WINDOW *win, int height)
       wcd_mvwaddstr(win,12,0,_("b                 go page backward."));
       wcd_mvwaddstr(win,13,0,_("u                 go half page up."));
       wcd_mvwaddstr(win,14,0,_("d                 go half page down."));
-      wcd_mvwaddstr(win,15,0,_("t                 switch centered mode on/off."));
-      wcd_mvwaddstr(win,16,0,_("T                 toggle between line drawing and ASCII characters."));
-      wcd_mvwaddstr(win,17,0,_("<Esc> or q        Abort."));
-      wcd_mvwaddstr(win,18,0,_("/                 Search forward."));
-      wcd_mvwaddstr(win,19,0,_("?                 Search backward."));
-      wcd_mvwaddstr(win,20,0,_("n                 Repeat latest / or ? search."));
-      wcd_mvwaddstr(win,21,0,_("<Enter>           Select directory."));
-      if (height > 22)
-        wcd_mvwaddstr(win,22,0,_("Press any key."));
+      wcd_mvwaddstr(win,16,0,_("Press any key."));
 
       prefresh(win,0,0,0,0,height-1,COLS-1);
       getch();
-      wclear(win);
+      werase(win);
+
+      wcd_mvwaddstr(win, 0,0,_("NAVIGATE MODE (2/2):"));
+      wcd_mvwaddstr(win, 1,0,_("A                 switch alternative tree navigation on/off."));
+      wcd_mvwaddstr(win, 2,0,_("t                 switch centered mode on/off."));
+      wcd_mvwaddstr(win, 3,0,_("T                 toggle between line drawing and ASCII characters."));
+      wcd_mvwaddstr(win, 4,0,_("m                 toggle between compact and wide tree."));
+      wcd_mvwaddstr(win, 5,0,_("<Esc> or q        Abort."));
+      wcd_mvwaddstr(win, 6,0,_("/                 Search forward."));
+      wcd_mvwaddstr(win, 7,0,_("?                 Search backward."));
+      wcd_mvwaddstr(win, 8,0,_("n                 Repeat latest / or ? search."));
+      wcd_mvwaddstr(win, 9,0,_("<Enter>           Select directory."));
+      wcd_mvwaddstr(win,11,0,_("Press any key."));
+
+      prefresh(win,0,0,0,0,height-1,COLS-1);
+      getch();
+      werase(win);
 
       wcd_mvwaddstr(win, 0,0,_("SEARCH MODE with wildcard and subdir support:"));
       wcd_mvwaddstr(win, 1,0,_("<Left>            go left."));
@@ -1679,7 +1850,7 @@ void showHelp(WINDOW *win, int height)
 
       prefresh(win,0,0,0,0,height-1,COLS-1);
       getch();
-      wclear(win);
+      werase(win);
 
       wcd_mvwaddstr(win, 0,0,_("ZOOMING:"));
       wcd_mvwaddstr(win, 1,0,_("z or i or CTRL-i  zoom in."));
@@ -1729,7 +1900,7 @@ dirnode pushZoom(dirnode zoomStack, dirnode curNode, int *ymax)
    dirnodeSetX(0,curNode);
    dirnodeSetY(0,curNode);
 
-   setXYTree(curNode);
+   setXYTree(curNode, &wcd_cwin.graphics_mode);
 
    *ymax = dirnodeGetY(getLastDescendant(curNode));
 
@@ -1765,7 +1936,7 @@ dirnode popZoom(dirnode zoomStack, dirnode curNode, int *ymax)
    newtop = endOfRecursionOfDirnodeParent(top);
    dirnodeSetX(0,newtop);
    dirnodeSetY(0,newtop);
-   setXYTree(newtop);
+   setXYTree(newtop, &wcd_cwin.graphics_mode);
 
    *ymax = dirnodeGetY(getLastDescendant(newtop));
 
@@ -1790,7 +1961,7 @@ void setFold(dirnode n, c3po_bool f, int *ymax)
          dirnodeSetFold(f,n);
 
          d = endOfRecursionOfDirnodeParent(n);
-         setXYTree(d);
+         setXYTree(d, &wcd_cwin.graphics_mode);
          *ymax = dirnodeGetY(getLastDescendant(d));
       }
    }
@@ -1838,7 +2009,7 @@ void setFold_sub(dirnode d, c3po_bool f, int *ymax)
       setFold_tree(d,&f);
       d->fold = false;
       top = endOfRecursionOfDirnodeParent(d);
-      setXYTree(top);
+      setXYTree(top, &wcd_cwin.graphics_mode);
       *ymax = dirnodeGetY(getLastDescendant(top));
       return;
    }
@@ -1884,7 +2055,7 @@ void condenseSubdirs(dirnode n, int *ymax)
    }
 
    d = endOfRecursionOfDirnodeParent(n);
-   setXYTree(d);
+   setXYTree(d, &wcd_cwin.graphics_mode);
    *ymax = dirnodeGetY(getLastDescendant(d));
 
    return;
@@ -1945,7 +2116,7 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
    }
 #endif
 
-   if (graphics_mode & WCD_GRAPH_COLOR)
+   if (wcd_cwin.graphics_mode & WCD_GRAPH_COLOR)
       initcolor();
 
    keypad(stdscr, TRUE);
@@ -1962,7 +2133,7 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
 
    wcd_cwin.inputWin = newpad(INPUT_WIN_HEIGHT,COLS);
 
-   if (graphics_mode & WCD_GRAPH_COLOR)
+   if (wcd_cwin.graphics_mode & WCD_GRAPH_COLOR)
    {
       colorbox (wcd_cwin.scrollWin, MENU_COLOR, 0);     /* BillyC add colors */
       colorbox (wcd_cwin.inputWin,  BODY_COLOR, 0);     /* BillyC add colors */
@@ -2064,14 +2235,30 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
            wcd_cwin.curNode = getFirstNodeInLevel(wcd_cwin.curNode,dirnodeGetY(wcd_cwin.curNode));
          break;
       case 'k':
-            wcd_cwin.curNode = getNodeCursUpNatural(wcd_cwin.curNode, graphics_mode);
+            if ((wcd_cwin.graphics_mode & WCD_GRAPH_COMPACT) && (wcd_cwin.graphics_mode & WCD_GRAPH_ALT))
+            {
+               y = dirnodeGetY(wcd_cwin.curNode) - 1;
+               if (y < 0)
+                  y = 0;
+               wcd_cwin.curNode = getFirstNodeInLevel(wcd_cwin.curNode,y);
+            }
+            else
+               wcd_cwin.curNode = getNodeCursUpNatural(wcd_cwin.curNode, wcd_cwin.graphics_mode);
          break;
       case 'j':
-            wcd_cwin.curNode = getNodeCursDownNatural(wcd_cwin.curNode, graphics_mode);
+            if ((wcd_cwin.graphics_mode & WCD_GRAPH_COMPACT) && (wcd_cwin.graphics_mode & WCD_GRAPH_ALT))
+            {
+               y = dirnodeGetY(wcd_cwin.curNode) + 1;
+               if (y > ymax)
+                  y = ymax;
+               wcd_cwin.curNode = getFirstNodeInLevel(wcd_cwin.curNode,y);
+            }
+            else
+               wcd_cwin.curNode = getNodeCursDownNatural(wcd_cwin.curNode, wcd_cwin.graphics_mode);
          break;
       case ',':
       case 'h':
-            wcd_cwin.curNode = getNodeCursLeft(wcd_cwin.curNode);
+            wcd_cwin.curNode = getNodeCursLeft(wcd_cwin.curNode, &ymax);
          break;
       case '.':
       case 'l':
@@ -2079,7 +2266,7 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
             {
                setFold(wcd_cwin.curNode,false,&ymax);
             }
-            wcd_cwin.curNode = getNodeCursRight(wcd_cwin.curNode, graphics_mode);
+            wcd_cwin.curNode = getNodeCursRight(wcd_cwin.curNode, wcd_cwin.graphics_mode);
          break;
       case '1': /* goto rootnode */
             wcd_cwin.curNode = endOfRecursionOfDirnodeParent(wcd_cwin.curNode);
@@ -2140,10 +2327,20 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
             wcd_cwin.curNode = findDirInCiclePrev(dirnodeGetName(wcd_cwin.curNode),wcd_cwin.curNode,1,ignore_case,ignore_diacritics);
      break;
       case 't':
-               graphics_mode ^= WCD_GRAPH_CENTER ;
+               wcd_cwin.graphics_mode ^= WCD_GRAPH_CENTER ;
          break;
       case 'T':
                wcd_cwin.graphics_mode ^= WCD_GRAPH_ASCII ;
+               if (wcd_cwin.graphics_mode & WCD_GRAPH_CJK)
+                  setXYTree(endOfRecursionOfDirnodeParent(wcd_cwin.curNode),&wcd_cwin.graphics_mode);
+         break;
+      case 'A':
+               wcd_cwin.graphics_mode ^= WCD_GRAPH_ALT ;
+         break;
+      case 'm':
+               wcd_cwin.graphics_mode ^= WCD_GRAPH_COMPACT ;
+               setXYTree(endOfRecursionOfDirnodeParent(wcd_cwin.curNode),&wcd_cwin.graphics_mode);
+               ymax = dirnodeGetY(getLastDescendant(wcd_cwin.curNode));
          break;
      default:
      break;
@@ -2226,19 +2423,35 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
 #ifdef KEY_A2
       case KEY_A2:  /*  Num-pad ARROW UP */
 #endif
-            wcd_cwin.curNode = getNodeCursUpNatural(wcd_cwin.curNode, graphics_mode);
+            if ((wcd_cwin.graphics_mode & WCD_GRAPH_COMPACT) && (wcd_cwin.graphics_mode & WCD_GRAPH_ALT))
+            {
+               y = dirnodeGetY(wcd_cwin.curNode) - 1;
+               if (y < 0)
+                  y = 0;
+               wcd_cwin.curNode = getFirstNodeInLevel(wcd_cwin.curNode,y);
+            }
+            else
+               wcd_cwin.curNode = getNodeCursUpNatural(wcd_cwin.curNode, wcd_cwin.graphics_mode);
          break;
       case KEY_DOWN: /* Arrow down */
 #ifdef KEY_C2
       case KEY_C2:   /* Num-pad  Arrow down */
 #endif
-            wcd_cwin.curNode = getNodeCursDownNatural(wcd_cwin.curNode, graphics_mode);
+            if ((wcd_cwin.graphics_mode & WCD_GRAPH_COMPACT) && (wcd_cwin.graphics_mode & WCD_GRAPH_ALT))
+            {
+               y = dirnodeGetY(wcd_cwin.curNode) + 1;
+               if (y > ymax)
+                  y = ymax;
+               wcd_cwin.curNode = getFirstNodeInLevel(wcd_cwin.curNode,y);
+            }
+            else
+               wcd_cwin.curNode = getNodeCursDownNatural(wcd_cwin.curNode, wcd_cwin.graphics_mode);
          break;
       case KEY_LEFT:
 #ifdef KEY_B1
       case KEY_B1:   /* Num-pad  Arrow left */
 #endif
-            wcd_cwin.curNode = getNodeCursLeft(wcd_cwin.curNode);
+            wcd_cwin.curNode = getNodeCursLeft(wcd_cwin.curNode, &ymax);
          break;
       case KEY_RIGHT:
 #ifdef KEY_B3
@@ -2248,7 +2461,7 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
             {
                setFold(wcd_cwin.curNode,false,&ymax);
             }
-            wcd_cwin.curNode = getNodeCursRight(wcd_cwin.curNode, graphics_mode);
+            wcd_cwin.curNode = getNodeCursRight(wcd_cwin.curNode, wcd_cwin.graphics_mode);
          break;
       case Key_CTRL ('g'):
          wcd_cwin.curNode = getLastDescendant(wcd_cwin.curNode);
@@ -2347,7 +2560,7 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
 
     ydiff -= (wcd_cwin.curNode->y);
 
-    if (graphics_mode & WCD_GRAPH_CENTER)
+    if (wcd_cwin.graphics_mode & WCD_GRAPH_CENTER)
        dataRefresh(0, 1);
     else
        dataRefresh(ydiff, 0);
@@ -2360,7 +2573,7 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
 
    printf("\n"); /* Extra newline for curses, pdcurses and when ncurses doesn't restore screen */
 
-#ifdef MSDOS
+#ifdef _WCD_DOSFS
    /* remove '/' before drive letter */
    if ((ptr != NULL)&&(*use_HOME)&&
          (*(ptr+1) != '/'))  /* UNC path */
