@@ -142,6 +142,7 @@ typedef char wcd_char;
 
 FILE *wcd_fopen(const char *filename, const char *m, int quiet);
 FILE *wcd_fopen_bom(const char *filename, const char *m, int quiet, int *bomtype);
+void wcd_fclose(FILE *fp, const char *filename, const char *m, const char *functionname);
 void finddirs(char *dir, size_t *offset, FILE *outfile, int *use_HOME, nameset exclude, int quiet);
 int read_treefile(char *filename, nameset bd, int silent);
 void rmDirFromList(char *string, nameset n);
@@ -167,9 +168,9 @@ int wcd_exit(nameset pm, nameset wm, nameset ef, nameset bd, nameset nfs, WcdSta
 void getCurPath(char *buffer, size_t size, int *use_HOME);
 
 #ifdef WCD_DOSBASH
-	void empty_wcdgo(char *go_file, int changedrive, char *drive, int use_GoScript);
+	void empty_wcdgo(char *go_file, int changedrive, char *drive, int use_GoScript, int verbose);
 #else
-	void empty_wcdgo(char *go_file, int use_GoScript);
+	void empty_wcdgo(char *go_file, int use_GoScript, int verbose);
 #endif
 int SpecialDir(const char *path);
 
