@@ -8,7 +8,7 @@ Author: Erwin Waterlander
 ======================================================================
 = Copyright                                                          =
 ======================================================================
-Copyright (C) 2002-2012 Erwin Waterlander
+Copyright (C) 2002-2013 Erwin Waterlander
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -285,6 +285,41 @@ int wcd_isSharePath (char* path)
    else
      return(0);
 }
+/*
+ * wcd_islink()
+ * Check if directory is a symbolic link.
+ * Returns: 1 if dir is a symlink, 0 if dir is not a symlink.
+ */
+
+int wcd_islink(const char *dir, int quiet)
+{
+   DWORD attrs;
+   DWORD dw;
+#ifdef WCD_UNICODE
+   static wchar_t wstr[DD_MAXPATH];
+
+   if (utf8towcs(wstr, dir, DD_MAXPATH) == (size_t)(-1))
+      attrs = INVALID_FILE_ATTRIBUTES;
+   else
+      attrs = GetFileAttributesW(wstr);
+#else
+   attrs = GetFileAttributes(dir);
+#endif
+
+   if (attrs == INVALID_FILE_ATTRIBUTES)
+   {
+      if ( !quiet )
+      {
+         dw = GetLastError();
+         wcd_printf(_("Wcd: error: wcd_islink: "));
+         PrintError(dw);
+      }
+      return(0);
+   }
+
+   return ((attrs & FILE_ATTRIBUTE_REPARSE_POINT) != 0);
+}
+
 
 /* If WCD_UNICODE is defined we assume that all multi-byte
  * strings are encoded in UTF-8.
