@@ -9,6 +9,8 @@
  *              Erwin Waterlander
  * Sep 8 2005:  Keep first two slashes also for Cygwin (UNC path)
  *              Erwin Waterlander
+ * Sep 15 2011: changed parameter 'lim' to size_t type.
+ *              Erwin Waterlander
  */
 
 #include <stdio.h>		/* For FILENAME_MAX */
@@ -17,10 +19,6 @@
 #include "tailor.h"
 #include "config.h"
 
-#ifdef OS2
-#undef UNIX
-#endif
-
 int wcd_is_slash(int c)
 {
 #if (defined(UNIX) && !defined(__CYGWIN__))  /* Oct 16 2001 */
@@ -50,14 +48,15 @@ static int is_term(int c)
   10. Don't fix path "."
 */
 
-void wcd_fixpath(char *in, int lim)
+void wcd_fixpath(char *in, size_t lim)
 {
-  int	i=0,drive_number= -1;
+  size_t i=0;
+  int	drive_number= -1;
   const char	*ip = in;
   char  *out;
   char	*op;
 
-  if (in == NULL)
+  if ((in == NULL) || (lim == 0))
    	return ;
 
   if ((out = (char *) malloc(lim)) == NULL)
@@ -68,7 +67,7 @@ void wcd_fixpath(char *in, int lim)
 
   op = out;
 
-#ifdef MSDOS
+#if defined(__MSDOS__) || defined(_WIN32) || defined(__OS2__)
   /* Add drive specification to output string (if present) */
   if (((*ip >= 'a' && *ip <= 'z') ||
        (*ip >= 'A' && *ip <= 'Z'))
@@ -83,7 +82,7 @@ void wcd_fixpath(char *in, int lim)
     {
       drive_number = *ip - 'A';
       if (*ip <= 'Z')
-	*op++ = drive_number + 'a';
+	*op++ = (char)(drive_number + 'a');
       else
 	*op++ = *ip;
       ++ip;
@@ -93,7 +92,7 @@ void wcd_fixpath(char *in, int lim)
   }
 #endif
 
-#if (defined(WIN32) || defined(__CYGWIN__))
+#if (defined(_WIN32) || defined(__CYGWIN__))
    /* Keep first "//" if present (UNC path) */
 	 if (wcd_is_slash(*ip) && wcd_is_slash(*(ip + 1)))
 	 {
